SystemOrganization addCategory: #'OpenVolunteerMarketPlace-Model'!
SystemOrganization addCategory: #'OpenVolunteerMarketPlace-View'!
SystemOrganization addCategory: #'OpenVolunteerMarketPlace-Controller'!
SystemOrganization addCategory: #'OpenVolunteerMarketPlace-Tests'!

TestCase subclass: #DatabaseTestCase
	instanceVariableNames: 'database'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Tests'!

!DatabaseTestCase methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 22:11'!
setUp
	database := Database new.! !

!DatabaseTestCase methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 22:11'!
testAddAndRemoveVolunteer
	|volunteer collection|
	volunteer :=  (OVMPVolunteer new name: 'name1'; email: 'email1@domain.com'; yourself).
	
	collection := database create: #volunteer.  
	
	database at: #volunteer add: volunteer.
	
	self should: [ (database collection: #volunteer  ) includes: volunteer. ].
	
	database at: #volunteer remove: volunteer.
	
	self shouldnt: [ (database collection: #volunteer) includes: volunteer. ].! !

!DatabaseTestCase methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 22:11'!
testFilterVolunteerByName
	|filter volunteer |
	database create: #volunteer.
	volunteer := OVMPVolunteer new name:'paulo'; email: 'paulo@domain.com'; yourself.
	database at: #volunteer add:(volunteer ).
	database at: #volunteer add:(OVMPVolunteer new name:'thiago'; email: 'thiago@domain.com'; yourself ).
	filter := database at: #volunteer filter: #name by: 'paulo'.
	self should: [filter size = 1].
	self should:[filter includes: volunteer ].! !

!DatabaseTestCase methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 22:12'!
testcreateCollection
	self should: [(database collection: #volunteer) isNil   ].
	database create: #volunteer.
	self shouldnt: [(database collection: #volunteer) isNil ].
! !

TestCase subclass: #InterfaceTestCase
	instanceVariableNames: 'ovmp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Tests'!

!InterfaceTestCase methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/21/2012 01:15'!
setUp
	ovmp := OpenVolunteerMarketplace new.
	ovmp initialize .! !

!InterfaceTestCase methodsFor: 'changing' stamp: 'pbm 10/21/2012 20:35'!
testAcceeptCandidate
	|project task candidate coordinator|
	
	candidate := ovmp newVolunteer: 'user'.
	coordinator := ovmp newVolunteer: 'coord'.
	project := ovmp newProject: 'project'  coordinatedBy: coordinator.
	task := (ovmp newTask: 'task' For: project coordinatedBy: coordinator).
	ovmp candidate: candidate toTask: task.
	(ovmp applicationsFor: task)  do:[:item | ovmp acceptApplication: item withPermissionsOf: coordinator   ].
	self should: [task volunteers includes: candidate].

	! !

!InterfaceTestCase methodsFor: 'changing' stamp: 'pbm 10/22/2012 14:55'!
testAcceptInvitation
	|project task candidate coordinator|
	
	candidate := ovmp newVolunteer: 'user'.
	coordinator := ovmp newVolunteer: 'coord'.  
	project := ovmp newProject: 'project'  coordinatedBy: coordinator .
	task := (ovmp newTask: 'task' For: project coordinatedBy: coordinator ).
	ovmp invite: candidate  to: task  by: coordinator.
	(ovmp invitationsTo: candidate ) do: [:invitation | ovmp acceptInvitation: invitation  by: candidate ].
	self should: [task volunteers includes: candidate].

	! !

!InterfaceTestCase methodsFor: 'adding' stamp: 'pbm 10/20/2012 22:35'!
testAddCoordinatorToProject
	|coordinator newcoordinator project|
	
	coordinator := ovmp newVolunteer: 'coordinator'.
	newcoordinator := ovmp newVolunteer: 'newcoordinator'.
	
	project := ovmp project: 'project001'.
	self should: [ project isNil  ].
	
	project := ovmp newProject: 'project001' coordinatedBy: coordinator .
	
	project := ovmp project: 'project001'.
	self shouldnt: [ project isNil  ].
	
	ovmp addCoordinator: newcoordinator for: project  withPermissionsOf: coordinator.
	
	self should: [ (ovmp project: 'project001') coordinators includes: coordinator ].
	self should: [ (ovmp project: 'project001') coordinators includes: newcoordinator ].
	
	! !

!InterfaceTestCase methodsFor: 'adding' stamp: 'pbm 10/20/2012 22:33'!
testAddNewProject
	|projectname user |
	
	projectname := 'project001'.
	self shouldnt: [ ovmp hasProject: projectname    ].
	
	user := ovmp newVolunteer: 'username'.
	ovmp newProject: projectname coordinatedBy: user.

	self should: [ ovmp hasProject: projectname   ].
	
! !

!InterfaceTestCase methodsFor: 'adding' stamp: 'pbm 10/20/2012 22:20'!
testAddNewSkill
	|user|
	
	user := ovmp volunteer: 'admin'.
	
	self should: [ (ovmp skill: 'skill') isNil ].
	
	ovmp newSkill: 'skill' withPermissionsOf: user.
	
	self shouldnt: [ (ovmp skill: 'skill') isNil ].
	
	ovmp removeSkill: (ovmp skill: 'skill') withPermissionsOf: user.! !

!InterfaceTestCase methodsFor: 'adding' stamp: 'pbm 10/20/2012 22:38'!
testAddNewVolunteer
	|username|
	
	username := 'testVolunteer'. 

	self shouldnt: [ ovmp hasVolunteer: username ].
	
	ovmp newVolunteer: username.
	
	self should: [ ovmp hasVolunteer: username ].
	
	
	
	! !

!InterfaceTestCase methodsFor: 'getting' stamp: 'pbm 10/21/2012 18:41'!
testAddRequiredSkillToTask
	|user project task skill |
	
	user := ovmp newVolunteer: 'user001'.
	project := ovmp newProject: 'project001' coordinatedBy: user.
	task := ovmp newTask: 'task001' For: project coordinatedBy: user.
	
	skill := (OVMPSkill new name: 'skill001').	
	ovmp addRequiredSkill: skill for: task coordinatedBy: user.
	
	self should: [ task skills includes: skill ].
	
	ovmp removeProject: project withPermissionsOf: user.! !

!InterfaceTestCase methodsFor: 'adding' stamp: 'pbm 10/21/2012 18:41'!
testAddTaskToProject
	|user project task|
	user := ovmp newVolunteer: 'username001'.
	project := ovmp newProject: 'project001' coordinatedBy: user.
	task := ovmp newTask: 'task001' For: project coordinatedBy: user.
	self should: [project tasks includes: task].
	ovmp removeProjectNamed: 'project001' withPermissionsOf: user. 
	
		! !

!InterfaceTestCase methodsFor: 'changing' stamp: 'pbm 10/21/2012 20:28'!
testCandidatingToTask
	|project task candidate|
	
	candidate := ovmp newVolunteer: 'user'.
	project := ovmp newProject: 'project'  coordinatedBy: candidate.
	task := (ovmp newTask: 'task' For: project coordinatedBy: candidate).
	ovmp candidate: candidate toTask: task.
	self should: [(ovmp candidatesFor: task) includes: candidate  ]. 
	! !

!InterfaceTestCase methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 14:20'!
testDeallocateVolunteerFromTask
	| task project user vol1 |
	user := ovmp newVolunteer: 'user001'.
	project := ovmp newProject: 'project001' coordinatedBy: user.
	task := OVMPTask forProject: project startingAt: Date today.
	vol1 := OVMPVolunteer forUsername: 'volunteer001'.
	task addVolunteer: vol1.
	self should: [ task volunteers size = 1 ].
	
	ovmp removeVolunteer: vol1 from: task withPermissionsOf: user.
	
	self should: [task volunteers size = 0].! !

!InterfaceTestCase methodsFor: 'getting' stamp: 'pbm 10/20/2012 22:49'!
testGetProject
	|projectname user |
	
	projectname := 'project001'.
	self shouldnt: [ ovmp hasProject: projectname].
	
	user :=  ovmp newVolunteer: 'username'.
	ovmp newProject: projectname coordinatedBy: user.

	self should: [ ovmp hasProject: projectname].
	
	self should: [(ovmp project: 'project001') coordinators includes: user].
! !

!InterfaceTestCase methodsFor: 'getting' stamp: 'pbm 10/20/2012 22:42'!
testGetVolunteer
	|username|
	
	username := 'testVolunteer'. 

	self shouldnt: [ ovmp hasVolunteer: username ].
	
	ovmp newVolunteer: username.
	
	self should: [ ovmp hasVolunteer: username ].
	
	self should: [(ovmp volunteer: username) username = username].
	
	
	
	
	! !

!InterfaceTestCase methodsFor: 'changing' stamp: 'pbm 10/22/2012 14:57'!
testInvitingToTask
	|project task candidate coordinator|
	
	candidate := ovmp newVolunteer: 'user'.
	coordinator := ovmp newVolunteer: 'coord'.  
	project := ovmp newProject: 'project'  coordinatedBy: coordinator .
	task := (ovmp newTask: 'task' For: project coordinatedBy: coordinator ).
	ovmp invite: candidate  to: task  by: coordinator.
	self should: [((ovmp invitationsTo: candidate) collect: [:invitation | invitation task]) includes: task].

	! !

!InterfaceTestCase methodsFor: 'changing' stamp: 'pbm 10/21/2012 20:41'!
testRejectCandidate
	|project task candidate coordinator|
	
	candidate := ovmp newVolunteer: 'user'.
	coordinator := ovmp newVolunteer: 'coord'.
	project := ovmp newProject: 'project'  coordinatedBy: coordinator.
	task := (ovmp newTask: 'task' For: project coordinatedBy: coordinator).
	ovmp candidate: candidate toTask: task.
	(ovmp applicationsFor: task)  do:[:item | ovmp rejectApplication: item withPermissionsOf: coordinator   ].
	(ovmp applicationsBy: candidate) do: [:item |
			self should:[item status = #rejected]]
	

	! !

!InterfaceTestCase methodsFor: 'changing' stamp: 'pbm 10/22/2012 15:03'!
testRejectInvitation
	|project task candidate coordinator|
	
	candidate := ovmp newVolunteer: 'user'.
	coordinator := ovmp newVolunteer: 'coord'.  
	project := ovmp newProject: 'project'  coordinatedBy: coordinator .
	task := (ovmp newTask: 'task' For: project coordinatedBy: coordinator ).
	ovmp invite: candidate  to: task  by: coordinator.
	(ovmp invitationsTo: candidate) do: [:invitation | ovmp reject: invitation by: candidate ].
	
	(ovmp invitationsTo: candidate) do: [:invitation | self should:[invitation status = #rejected  ]].

	! !

!InterfaceTestCase methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 14:07'!
testRemoveProjectWithTasks
	|project task1 task2 user |
	
	project := ovmp newProject: 'project002' coordinatedBy: user.
	task1 := OVMPTask 	forProject: project startingAt: (Date today ).
	task1 name: 'task001'.
	task2 := OVMPTask forProject: project startingAt: (Date today ).	
	task2 name: 'task002'.
	
	self should: [(ovmp project: 'project002') tasks size = 2].
	
	ovmp removeProject: project withPermissionsOf: (ovmp volunteer: 'admin').
	
	self shouldnt: [ovmp tasks includes: task1].
	self shouldnt: [ovmp tasks includes: task2].! !

!InterfaceTestCase methodsFor: 'removing' stamp: 'ThiagoFurtado 10/22/2012 17:01'!
testRemoveTaskFromProject

	| user project task task2 task3 |
	user := ovmp newVolunteer: 'coordinator'.
	project := ovmp newProject: 'project' coordinatedBy: user.
	
	task := ovmp newTask: 'task' For: project coordinatedBy: user.
	task2 := ovmp newTask: 'task2' For: project coordinatedBy: user.
	"task3 := ovmp newTask: 'task3' For: project coordinatedBy: user.
	task3 addDependencyOn: task2."
	
	self should: [project tasks size = 2].
	
	ovmp removeTask: task from: project withPermissionsOf: user.
	
	self should: [project tasks size = 1].
	
	ovmp removeTask: task2 from: project withPermissionsOf: (ovmp volunteer: 'admin').
	
	self should: [project tasks size = 0].! !

!InterfaceTestCase methodsFor: 'removing' stamp: 'ThiagoFurtado 10/22/2012 17:10'!
testRemoveTaskFromProject2

	| user project task task2 task3 |
	user := ovmp newVolunteer: 'coordinator'.
	project := ovmp newProject: 'project' coordinatedBy: user.
	
	task := ovmp newTask: 'task' For: project coordinatedBy: user.
	task2 := ovmp newTask: 'task2' For: project coordinatedBy: user.
	task3 := ovmp newTask: 'task3' For: project coordinatedBy: user.
	task3 addDependencyOn: task2.
	
	self should: [project tasks size = 3].
	
	ovmp removeTask: task from: project withPermissionsOf: user.
	
	self should: [project tasks size = 2].
	
	ovmp removeTask: task2 from: project withPermissionsOf: (ovmp volunteer: 'admin').
	
	self should: [project tasks size = 1].
	self should: [ task3 dependsOn size = 0].! !

!InterfaceTestCase methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 13:51'!
testRemoveVolunteerFromApplication
	|vol|
	vol := ovmp newVolunteer: 'volunteer001'.
	
	self should: [(ovmp volunteer: 'volunteer001') = vol ].
	
	ovmp removeVolunteer: vol withPermissionsOf: (ovmp volunteer: 'admin').
	
	self  should: [(ovmp volunteer: 'volunteer001') = nil].! !

!InterfaceTestCase methodsFor: 'getting' stamp: 'ThiagoFurtado 10/22/2012 15:20'!
testTasksSuggestion
	|admin project user task suggestion|
	admin := ovmp volunteer: 'admin'.  
	ovmp newSkill: 'skill1' withPermissionsOf: admin.
	ovmp newSkill: 'skill2' withPermissionsOf: admin.
	ovmp newSkill: 'skill3' withPermissionsOf: admin.
	
	user := ovmp newVolunteer: 'volunteer'.
	
	project := ovmp newProject: 'project' coordinatedBy:  user.
	
	task := ovmp newTask: 'task1' For: project coordinatedBy: user.
	task addSkill: (ovmp skill: 'skill1').
	task addSkill: (ovmp skill: 'skill2').
	
	task := ovmp newTask: 'task2'For: project coordinatedBy: user.
	task addSkill: (ovmp skill: 'skill1').
	task addSkill: (ovmp skill: 'skill3').
	
	user addSkill: (ovmp skill: 'skill1').
	user addSkill: (ovmp skill: 'skill3').

	suggestion := ovmp suggestedTasks: user.
	
	
	self should: [suggestion size == 1].
	self should: [suggestion includes: task]  ! !

!InterfaceTestCase methodsFor: 'getting' stamp: 'ThiagoFurtado 10/21/2012 02:10'!
testVolunteerSuggestion
	|admin project user task suggestion|
	admin := ovmp volunteer: 'admin'.  
	ovmp newSkill: 'skill1' withPermissionsOf: admin.
	ovmp newSkill: 'skill2' withPermissionsOf: admin.
	ovmp newSkill: 'skill3' withPermissionsOf: admin.
	
	user := ovmp newVolunteer: 'volunteer'.
	
	project := ovmp newProject: 'project' coordinatedBy:  user.
	
	task := ovmp newTask: 'task1' For: project coordinatedBy: user.
	task addSkill: (ovmp skill: 'skill1').
	task addSkill: (ovmp skill: 'skill2').
		
	user addSkill: (ovmp skill: 'skill1').
	user addSkill: (ovmp skill: 'skill3').
	
	user addSkill: (ovmp skill: 'skill1').
	user addSkill: (ovmp skill: 'skill2').

	suggestion := ovmp suggestedVolunteers: task.  
	
	Transcript show: 'suggestion size', (suggestion size) asString .  
	
	self should: [suggestion size == 1].
	self should: [suggestion includes: user]  ! !

WAComponent subclass: #OVMPContentView
	instanceVariableNames: 'searchBox projectListView'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

!OVMPContentView methodsFor: 'initialization' stamp: 'ThiagoFurtado 10/3/2012 12:16'!
initialize 
	super initialize .
	searchBox := OVMPSeachProjectView new.
	projectListView := OVMPProjectListView new.! !

!OVMPContentView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 12:13'!
renderContentOn: html

	html section id: 'content';
		with: [
			html render: searchBox.
			html render: projectListView.
			].! !

WAComponent subclass: #OVMPLoginView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

!OVMPLoginView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 09:01'!
renderContentOn: html

			html div
				id: 'login';
				with: [
					html form 
					id: 'formlogin';
					with: [
						html fieldSet
							class: 'up';
							with: [
								"html label for: 'email'; with: 'email/username'. "html textInput id: 'email'; type: 'text'; placeholder: 'user/email'.
								"html label for: 'pwd'; with: 'password'." html textInput id: 'pwd';type: 'password'; placeholder: 'password'.
								html submitButton id: 'enter'; class: 'btn'; with: 'Enter'.
								]
						 ].
			].! !

WAComponent subclass: #OVMPNewProjectView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

!OVMPNewProjectView methodsFor: 'project' stamp: 'ThiagoFurtado 10/3/2012 21:01'!
addProject
	self answer: true.! !

!OVMPNewProjectView methodsFor: 'project' stamp: 'ThiagoFurtado 10/3/2012 20:48'!
cancelAdding
	self answer: false.! !

!OVMPNewProjectView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 15:25'!
renderContentOn: html

	html article
		id: 'projectadding';
		with: [self renderProjectForm: html.]
		
											! !

!OVMPNewProjectView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/4/2012 00:04'!
renderProjectForm: html

	html article
		id: 'adding';
		with: [ 			
					html form
						id: 'addform';
						with: [ 
									html fieldSet
										class: 'addclass';
										with: [ 
													html textInput
														id: 'projectname';
														type: 'text';
														placeholder: 'Enter a project name'.
													html textInput 
														id: 'projectURL';
														placeholder: 'Enter a URL'.
													html textArea 
														id: 'projectdescription';
														placeholder: 'Enter a description'.
														"rows: 5;
														columns: 100."
													html submitButton 
														id: 'cancelbtn';
														class: 'btn';
														on: #cancelAdding of: self;
														with: 'Cancel'.
													html submitButton 
														id: 'addbtn';
														class: 'btn';
														on: #addProject of: self; 
														with: 'Add'.
												]	
							]				
				]					! !

WAComponent subclass: #OVMPProjectListView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

!OVMPProjectListView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 13:01'!
renderContentOn: html
	| projects |
	
	"just to test"
	projects := Array new.
	
	html div id: 'list';
		with: [
			
			html unorderedList: [
					projects do: [ :project |
							html listItem: [ html render: project. ].
						]
				]			
			
			].! !

WAComponent subclass: #OVMPProjectView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

WAComponent subclass: #OVMPSeachProjectView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

!OVMPSeachProjectView methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/3/2012 14:48'!
addNewProject
	| newProjectView |
	newProjectView := OVMPNewProjectView new.
	
	self call: newProjectView.! !

!OVMPSeachProjectView methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/3/2012 14:38'!
addNewProject: aProject

	self call: (OVMPNewProjectView new yourself)! !

!OVMPSeachProjectView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 21:28'!
renderContentOn: html
	html article
		id: 'projectmgmt';
		with: [self renderFields: html.]
		
											
	! !

!OVMPSeachProjectView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 21:28'!
renderFields: html

	html article
		id: 'search';
		with: [ 			
					html form
						id: 'searchform';
						with: [ 
									html fieldSet
										class: 'searchclass';
										with: [ 
													html submitButton 
														id: 'addproject';
														class: 'btn';
														callback: [self addNewProject ];																
														with: 'New'.
													html textInput
														id: 'projectfield';
														type: 'text';
														placeholder: 'Enter the searching project name'.
													html submitButton
														id: 'enter';
														class: 'btn';
														with: 'Search' ] ] ]! !

WAComponent subclass: #OVMPView
	instanceVariableNames: 'loginComponent centerComponent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-View'!

!OVMPView class methodsFor: 'class initialization' stamp: 'ThiagoFurtado 9/22/2012 14:31'!
initialize

	WAAdmin register: self asApplicationAt: 'openvolunteermarketplace' .! !

!OVMPView methodsFor: 'accessing' stamp: 'ThiagoFurtado 9/29/2012 00:22'!
children 
	^ Array with: loginComponent with: searchProjectBox with: centerComponent.! !

!OVMPView methodsFor: 'initialization' stamp: 'ThiagoFurtado 10/3/2012 12:14'!
initialize 
	super initialize .
	loginComponent := OVMPLoginView new.
	centerComponent := OVMPContentView new.! !

!OVMPView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 12:13'!
renderBodyContentOn: html

	html section id: 'content';
		with: [
			html render: centerComponent.
			].! !

!OVMPView methodsFor: 'rendering' stamp: 'ThiagoFurtado 9/28/2012 23:39'!
renderContentOn: html

	html div 
		id: 'main';
		with: [ self renderHeaderOn: html .
			    self renderBodyContentOn: html ].! !

!OVMPView methodsFor: 'rendering' stamp: 'ThiagoFurtado 10/3/2012 09:02'!
renderHeaderOn: html
	html header
		id: 'header';
		with: [ 
					html div
						id: 'in-header';
						with: [ 
									html heading
										level: 1;
										with: [ 
													html anchor
														url: 'http://localhost:8080/openvolunteermarketplace';
														with: 'Open Volunteer MarketPlace' ].
									self renderLoginForm: html ] ]! !

!OVMPView methodsFor: 'rendering' stamp: 'ThiagoFurtado 9/28/2012 23:48'!
renderLoginForm: html

	html render: loginComponent.! !

!OVMPView methodsFor: 'rendering' stamp: 'pbm 10/5/2012 14:31'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	anHtmlRoot title: 'Open Volunteer Marketplace'.
	anHtmlRoot stylesheet url: 'http://localhost/openvolunteermarketplace/styles/style.css'! !

Object subclass: #Database
	instanceVariableNames: 'Collections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Controller'!

!Database methodsFor: 'accessing collections' stamp: 'pbm 10/20/2012 22:07'!
at: aSymbol add: aValue
	^(self collection: aSymbol) add: aValue   ! !

!Database methodsFor: 'accessing collections' stamp: 'pbm 10/20/2012 23:35'!
at: aSymbol filter: anAttribute by: aValue
	|message collection|
	message := Message new setSelector: anAttribute arguments: #(); yourself .
	collection := self collection: aSymbol.
	^collection select: [:item | (message sendTo: item) = aValue].  
! !

!Database methodsFor: 'accessing collections' stamp: 'pbm 10/20/2012 22:08'!
at: aSymbol remove: aValue
	(self collection: aSymbol) remove: aValue   ! !

!Database methodsFor: 'accessing collections' stamp: 'pbm 10/20/2012 22:08'!
at: aSymbol removeAll: aCollection
	(self collection: aSymbol) removeAll: aCollection ! !

!Database methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:08'!
collection: aSymbol
	^Collections at: aSymbol ifAbsent: nil! !

!Database methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:09'!
create: aSymbol
	(Collections isNil ) ifTrue: [self initialize ].  
	Collections add: aSymbol -> (Set new).! !

!Database methodsFor: 'class initialization' stamp: 'pbm 10/20/2012 22:09'!
initialize 
	super initialize .
	Collections := Dictionary new.! !

Object subclass: #OVMPComment
	instanceVariableNames: 'datetime volunteer text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

Object subclass: #OVMPProject
	instanceVariableNames: 'name startDate endDate state tasks coordinators creator url description comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPProject class methodsFor: 'as yet unclassified' stamp: 'pbm 10/19/2012 20:48'!
forName: aName
	^self new name: aName; yourself! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 15:30'!
addComment: aComment
	comments add: aComment.! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 15:31'!
addComments: aCollection
	comments addAll: aCollection.! !

!OVMPProject methodsFor: 'updating' stamp: 'ThiagoFurtado 10/20/2012 20:06'!
addCoordinator: aVolunteer
	( coordinators isNil ) ifTrue: [self initialize ].
	coordinators add: aVolunteer ! !

!OVMPProject methodsFor: 'updating' stamp: 'ThiagoFurtado 9/26/2012 11:14'!
addTask: aTask
	tasks add: aTask.! !

!OVMPProject methodsFor: 'updating' stamp: 'ThiagoFurtado 9/26/2012 11:15'!
addTasks: aTaskArray
	tasks addAll: aTaskArray.! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 15:31'!
comments
	^ comments.! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/14/2012 17:46'!
coordinators
	^ coordinators ! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 16:19'!
creator
	^ creator.! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 16:19'!
creator: aVolunteer
	creator := aVolunteer.! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/22/2012 13:32'!
description 
	^description ! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/22/2012 13:33'!
description: aString
	description := aString ! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/14/2012 17:43'!
endDate
	^ endDate ! !

!OVMPProject methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:43'!
endDate: aDate
	endDate := aDate! !

!OVMPProject methodsFor: 'initialize-release' stamp: 'ThiagoFurtado 10/22/2012 14:46'!
initialize
	coordinators := Set new.
	tasks := SortedCollection new sort:[:first :second | first startDate <= second startDate ]; yourself .
	self startDate: Date today .
	self state: (OVMPProjectRunning new).
	comments := SortedCollection new sort:[:first :second | first datetime <= second datetime ]; yourself .! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 9/26/2012 11:12'!
name
	^ name.! !

!OVMPProject methodsFor: 'updating' stamp: 'ThiagoFurtado 9/26/2012 11:12'!
name: aName
	name := aName.! !

!OVMPProject methodsFor: 'printing' stamp: 'pbm 10/22/2012 14:25'!
printString
	|return|
	return := name, ' start date: ', startDate, ' end date:', endDate, ' state: ', state, String cr.
	return := return, 'coordinators: '.
	coordinators do: [:coordinator | return := return, coordinator name, ', '].
	return := return, String cr.
	return := return, 'tasks:', String cr.
	tasks do:[:task | return := return, '* ', task printString, String cr].
	^return! !

!OVMPProject methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:46'!
removeCoordinator: aCoordinator
	coordinators remove: aCoordinator   ! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 9/26/2012 11:13'!
startDate
	^ startDate.! !

!OVMPProject methodsFor: 'updating' stamp: 'ThiagoFurtado 9/26/2012 11:12'!
startDate: aDate
	startDate := aDate.! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/14/2012 17:44'!
state
	^ state! !

!OVMPProject methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:44'!
state: aState
	state := aState ! !

!OVMPProject methodsFor: 'accessing' stamp: 'ThiagoFurtado 9/26/2012 11:13'!
tasks
	^ tasks.! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/22/2012 13:32'!
url
	^url! !

!OVMPProject methodsFor: 'accessing' stamp: 'pbm 10/22/2012 13:32'!
url: aString
	url := aString ! !

Object subclass: #OVMPProjectState
	instanceVariableNames: 'project internalState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

OVMPProjectState subclass: #OVMPProjectCanceled
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPProjectCanceled methodsFor: 'as yet unclassified' stamp: 'pbm 10/22/2012 14:21'!
printString 
	^#cancelled! !

!OVMPProjectCanceled methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:23'!
run
	OVMPProjectRunning forTask: project.  ! !

OVMPProjectState subclass: #OVMPProjectFinished
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPProjectFinished methodsFor: 'as yet unclassified' stamp: 'pbm 10/22/2012 14:21'!
printString 
	^#finished! !

OVMPProjectState subclass: #OVMPProjectRunning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPProjectRunning methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:24'!
cancel
	OVMPProjectCanceled forTask: project. ! !

!OVMPProjectRunning methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:24'!
finish
	OVMPProjectFinished forTask: project.  ! !

!OVMPProjectRunning methodsFor: 'as yet unclassified' stamp: 'pbm 10/22/2012 14:23'!
printString 
	^#running! !

!OVMPProjectState class methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:23'!
forProject: aProject
	aProject state: self new.! !

!OVMPProjectState methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:20'!
cancel
	self subclassResponsibility .! !

!OVMPProjectState methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:20'!
finish 
	self subclassResponsibility .! !

!OVMPProjectState methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/22/2012 14:20'!
run
	self subclassResponsibility .! !

Object subclass: #OVMPRequest
	instanceVariableNames: 'task status comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

OVMPRequest subclass: #OVMPApplication
	instanceVariableNames: 'volunteer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPApplication class methodsFor: 'initialize-release' stamp: 'pbm 10/21/2012 19:40'!
to: aTask by: aVolunteer
	^self new volunteer: aVolunteer ; task: aTask.! !

!OVMPApplication methodsFor: 'accessing' stamp: 'pbm 10/21/2012 19:29'!
volunteer
	^volunteer ! !

!OVMPApplication methodsFor: 'accessing' stamp: 'pbm 10/21/2012 19:29'!
volunteer: aVolunteer
	volunteer := aVolunteer ! !

OVMPRequest subclass: #OVMPInvitation
	instanceVariableNames: 'inviting invitee'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPInvitation class methodsFor: 'initialize-release' stamp: 'pbm 10/22/2012 14:43'!
to: aVolunteer for: aTask by: aCoordinator
	^super new invitee: aVolunteer; task: aTask; inviting: aCoordinator; yourself 
	! !

!OVMPInvitation methodsFor: 'accessing' stamp: 'pbm 10/20/2012 19:29'!
invitee 
	^invitee ! !

!OVMPInvitation methodsFor: 'accessing' stamp: 'pbm 10/20/2012 19:30'!
invitee: anInvitee
	invitee := anInvitee ! !

!OVMPInvitation methodsFor: 'accessing' stamp: 'pbm 10/20/2012 19:30'!
inviting
	^inviting
	! !

!OVMPInvitation methodsFor: 'accessing' stamp: 'pbm 10/20/2012 19:30'!
inviting: anInviting
	inviting := anInviting 
	! !

!OVMPRequest methodsFor: 'initialize-release' stamp: 'pbm 10/22/2012 11:52'!
initialize
	status := #pending.! !

!OVMPRequest methodsFor: 'accessing' stamp: 'pbm 10/22/2012 11:52'!
reject
	status := #rejected! !

!OVMPRequest methodsFor: 'accessing' stamp: 'pbm 10/22/2012 11:52'!
reject: aComment
	status := #rejected.
	comment := aComment ! !

!OVMPRequest methodsFor: 'accessing' stamp: 'pbm 10/22/2012 11:52'!
status
	^status ! !

!OVMPRequest methodsFor: 'accessing' stamp: 'pbm 10/22/2012 11:53'!
task
	^task! !

!OVMPRequest methodsFor: 'accessing' stamp: 'pbm 10/22/2012 11:53'!
task: aTask
	task := aTask ! !

Object subclass: #OVMPSkill
	instanceVariableNames: 'name description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPSkill class methodsFor: 'as yet unclassified' stamp: 'ThiagoFurtado 10/20/2012 21:33'!
forName: aName
	^ self new name: aName; yourself .! !

!OVMPSkill methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 15:47'!
description
	^ description.! !

!OVMPSkill methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 15:47'!
description: aDescription
	description := aDescription.! !

!OVMPSkill methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 15:47'!
name
	^ name.! !

!OVMPSkill methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 15:47'!
name: aName
	name := aName.! !

!OVMPSkill methodsFor: 'printing' stamp: 'pbm 10/22/2012 13:07'!
printString
	^name! !

Object subclass: #OVMPTask
	instanceVariableNames: 'name volunteers skills startDate state estimatedDuration endDate project dependsOn desiredConclusionDate comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPTask class methodsFor: 'instance creation' stamp: 'ThiagoFurtado 10/21/2012 16:00'!
forProject: aProject startingAt: aDate
	|tmptask|
	tmptask := (self new startDate: aDate; project: aProject; yourself).
	tmptask state: (OVMPTaskOpen new).
	aProject addTask: tmptask.
	^tmptask.! !

!OVMPTask class methodsFor: 'instance creation' stamp: 'pbm 10/22/2012 13:44'!
named: aName forProject: aProject startingAt: aDate
	|tmptask|
	tmptask := (self new name: aName; startDate: aDate; project: aProject; yourself).
	tmptask state: (OVMPTaskOpen new).
	aProject addTask: tmptask.
	^tmptask.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 15:32'!
addComment: aComment
	comments add: aComment.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:52'!
addDependenciesOn: tasks
	dependsOn addAll: tasks.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:51'!
addDependencyOn: task
	dependsOn add: task.! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/14/2012 18:02'!
addSkill: aSkill
	skills add: aSkill     ! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/22/2012 13:21'!
addSkills: aCollection
	skills addAll: aCollection ! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:59'!
addVolunteer: aVolunteer
	volunteers add: aVolunteer   ! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/21/2012 18:59'!
cancel
	state cancel.! !

!OVMPTask methodsFor: 'testing' stamp: 'pbm 10/20/2012 18:39'!
checkCoveredSkills: aCollection
	"returns the skills from aCollection that are required by this task"
	^aCollection select: [:item| skills includes: item ].! !

!OVMPTask methodsFor: 'testing' stamp: 'pbm 10/20/2012 18:38'!
checkUncoveredSkills: aCollection
	"returns the skills not included in aCollection"
	^skills reject: [:item| aCollection includes: item  ].! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 15:33'!
comments
	^ comments.! !

!OVMPTask methodsFor: 'accessing' stamp: 'pbm 10/22/2012 15:44'!
coordinators 
	^project coordinators ! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:40'!
dependsOn
	^ dependsOn.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:43'!
desiredConclusionDate
	^ desiredConclusionDate.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:43'!
desiredConclusionDate: date
	desiredConclusionDate := date.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:06'!
estimatedDuration
	^ self estimatedDuration.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 14:06'!
estimatedDuration: time
	estimatedDuration := time.! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/21/2012 18:59'!
finish
	state finish.! !

!OVMPTask methodsFor: 'testing' stamp: 'pbm 10/20/2012 18:16'!
hasVolunteer
	^volunteers isEmpty not! !

!OVMPTask methodsFor: 'initialize-release' stamp: 'ThiagoFurtado 10/22/2012 14:46'!
initialize 
	volunteers := Set new.
	skills := Set new.
	state := OVMPTaskOpen new.
	dependsOn := Set new.
	comments := SortedCollection new sort:[:first :second | first datetime <= second datetime ]; yourself .! !

!OVMPTask methodsFor: 'testing' stamp: 'pbm 10/20/2012 23:27'!
isOpen
	^(state class = OVMPTaskOpen )! !

!OVMPTask methodsFor: 'accessing' stamp: 'pbm 10/14/2012 17:56'!
name
	^ name! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:49'!
name: aName
	name := aName! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/21/2012 18:58'!
open
	state open.! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/21/2012 18:59'!
postpone
	state postpone.! !

!OVMPTask methodsFor: 'printing' stamp: 'pbm 10/22/2012 14:10'!
printString
	|return|
	return := name, ' startDate: ', startDate, ' state: ', state, ' estimated duration: ', estimatedDuration, ' conclusion: ', endDate, String cr.
	return := return, String tab, 'required skills: '.
	skills do: [:skill| return := return, skill name, ' '].
	return := return, String cr.
	return := return, String tab, 'depends on: '.
	dependsOn do: [:task | return := return, task name, ' '].
	return := return, String cr.
	return := return, String tab, 'volunteers: '.
	volunteers do: [:volunteer | return := return, String cr, String tab, String tab, volunteer printString ].
	^return! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 12:20'!
project
	^ project.! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/20/2012 12:20'!
project: aProject
	project := aProject .! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/14/2012 18:02'!
removeSkill: aSkill
	skills remove: aSkill   ! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/14/2012 18:00'!
removeVolunteer: aVolunteer
	volunteers remove: aVolunteer   ! !

!OVMPTask methodsFor: 'testing' stamp: 'pbm 10/14/2012 21:44'!
requiresSkill: aSkill
	^skills includes: aSkill     ! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/21/2012 19:00'!
run
	state run.! !

!OVMPTask methodsFor: 'accessing' stamp: 'pbm 10/14/2012 18:01'!
skills
	^ skills ! !

!OVMPTask methodsFor: 'accessing' stamp: 'pbm 10/14/2012 18:33'!
startDate
	^ startDate ! !

!OVMPTask methodsFor: 'updating' stamp: 'pbm 10/14/2012 18:34'!
startDate: aDate
	startDate := aDate ! !

!OVMPTask methodsFor: 'accessing' stamp: 'pbm 10/20/2012 23:40'!
state
	^state! !

!OVMPTask methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/21/2012 16:04'!
state: aState
	state := aState.! !

!OVMPTask methodsFor: 'accessing' stamp: 'pbm 10/14/2012 17:57'!
volunteers
	^ volunteers ! !

Object subclass: #OVMPTaskState
	instanceVariableNames: 'task internalState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

OVMPTaskState subclass: #OVMPTaskCanceled
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPTaskCanceled methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 23:26'!
open
	OVMPTaskOpen forTask: task.  ! !

!OVMPTaskCanceled methodsFor: 'as yet unclassified' stamp: 'pbm 10/21/2012 19:20'!
printString
	^#canceled! !

OVMPTaskState subclass: #OVMPTaskFinished
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPTaskFinished methodsFor: 'as yet unclassified' stamp: 'pbm 10/21/2012 19:21'!
printString
	^#finished! !

OVMPTaskState subclass: #OVMPTaskOpen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPTaskOpen methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:12'!
postpone
	OVMPTaskPostponed forTask: task  ! !

!OVMPTaskOpen methodsFor: 'as yet unclassified' stamp: 'pbm 10/21/2012 19:21'!
printString
	^#open! !

!OVMPTaskOpen methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:18'!
run
	self canRun.
	OVMPTaskRunning forTask: task.! !

OVMPTaskState subclass: #OVMPTaskPostponed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPTaskPostponed methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 23:26'!
open
	OVMPTaskOpen forTask: task.  ! !

!OVMPTaskPostponed methodsFor: 'as yet unclassified' stamp: 'pbm 10/21/2012 19:21'!
printString
	^#postponed! !

!OVMPTaskPostponed methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:18'!
run
	self canRun.
	OVMPTaskRunning forTask: task.  ! !

OVMPTaskState subclass: #OVMPTaskRunning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPTaskRunning methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:10'!
finish
	OVMPTaskFinished forTask: task.  ! !

!OVMPTaskRunning methodsFor: 'as yet unclassified' stamp: 'pbm 10/21/2012 19:21'!
printString
	^#running! !

!OVMPTaskState class methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:05'!
forTask: aTask
	aTask state: self new.! !

!OVMPTaskState methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:18'!
canRun
	task hasVolunteer ifFalse: [self error: 'cannot start a task without volunteers'].! !

!OVMPTaskState methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 18:07'!
cancel
	OVMPTaskCanceled forTask: task.  ! !

!OVMPTaskState methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 17:58'!
finish
	self subclassResponsibility ! !

!OVMPTaskState methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 17:57'!
open
	self subclassResponsibility ! !

!OVMPTaskState methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 17:58'!
postpone
	self subclassResponsibility ! !

!OVMPTaskState methodsFor: 'as yet unclassified' stamp: 'pbm 10/20/2012 17:57'!
run
	self subclassResponsibility ! !

Object subclass: #OVMPVolunteer
	instanceVariableNames: 'username password email name birthday skills admin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Model'!

!OVMPVolunteer class methodsFor: 'instance creation' stamp: 'ThiagoFurtado 10/21/2012 01:25'!
forUsername: aUsername
	^self new username: aUsername; admin: false; yourself.! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'ThiagoFurtado 9/26/2012 13:11'!
addSkill: aSkill
	skills add: aSkill.! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'ThiagoFurtado 9/26/2012 13:12'!
addSkills: aSkillArray
	 skills addAll: aSkillArray.! !

!OVMPVolunteer methodsFor: 'initialize-release' stamp: 'ThiagoFurtado 10/20/2012 13:46'!
admin: aBoolean
	admin := aBoolean.! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 13:34'!
birthday
	^ birthday.! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/22/2012 13:34'!
birthday: aDate
	birthday := aDate.! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'pbm 10/14/2012 19:12'!
email 
	^email ! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'pbm 10/14/2012 19:13'!
email: anEmail
 email := anEmail ! !

!OVMPVolunteer methodsFor: 'testing' stamp: 'pbm 10/14/2012 21:45'!
hasSkill: aSkill
	^ skills includes: aSkill   ! !

!OVMPVolunteer methodsFor: 'initialize-release' stamp: 'pbm 10/20/2012 19:36'!
initialize 
	skills := Set new.
! !

!OVMPVolunteer methodsFor: 'initialize-release' stamp: 'ThiagoFurtado 10/20/2012 13:46'!
isAdmin
	^ admin.! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'pbm 10/14/2012 17:28'!
name
	"get person's name"
	^ name.! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:28'!
name: aName
	"set person's name"
	name := aName.! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'pbm 10/14/2012 19:11'!
password 
	^ password ! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'pbm 10/14/2012 19:12'!
password: aPassword
	password := aPassword ! !

!OVMPVolunteer methodsFor: 'printing' stamp: 'pbm 10/22/2012 13:53'!
printString
	|return|
	return := name, ' - ', email, ' ('.
	skills do:[:skill | return := return, skill name, ' ' ].
	return := return,')'.
	^return! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'pbm 10/14/2012 19:13'!
removeSkill: aSkill
	skills remove: aSkill     ! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'ThiagoFurtado 9/26/2012 13:11'!
skills
	^ skills.! !

!OVMPVolunteer methodsFor: 'accessing' stamp: 'pbm 10/14/2012 19:11'!
username
	^ username ! !

!OVMPVolunteer methodsFor: 'updating' stamp: 'pbm 10/14/2012 17:29'!
username: aUsername
	"set person's username"

	username := aUsername.! !

Object subclass: #OpenVolunteerMarketplace
	instanceVariableNames: 'database'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenVolunteerMarketPlace-Controller'!

!OpenVolunteerMarketplace methodsFor: 'changing' stamp: 'pbm 10/22/2012 15:49'!
accept: aVolunteer toTask: aTask by: aCoordinator
	self checkPermission: aTask with: aCoordinator.
	aTask addVolunteer: aVolunteer.  ! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/21/2012 20:13'!
acceptApplication: anApplication withPermissionsOf: aCoodinador
	self accept: anApplication volunteer toTask: anApplication task by: aCoodinador.
	database at: #application remove: anApplication.
	! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/22/2012 14:55'!
acceptInvitation: anInvitation by: anInvitee
	(anInvitation invitee = anInvitee) ifFalse: [self error:'operation not allowed'].
	anInvitation task addVolunteer: anInvitee.
	database at: #invitation remove: anInvitation .  
	! !

!OpenVolunteerMarketplace methodsFor: 'adding' stamp: 'pbm 10/22/2012 15:50'!
addCoordinator: newCoordinator for: aProject withPermissionsOf: aCoordinator
	| tmpProject|
	
	tmpProject := self project: ( aProject name ) .
	
	( tmpProject isNil )
		ifTrue: [
			self error: 'There is no project with name ', ( aProject name ). 
			]	
		ifFalse: [
			self checkPermission: aProject with: aCoordinator.
			(self project: ( aProject name )) addCoordinator: newCoordinator .
			]
! !

!OpenVolunteerMarketplace methodsFor: 'changing' stamp: 'ThiagoFurtado 10/22/2012 14:53'!
addDependenciesOn: dependables to: aTask

	aTask addDependenciesOn: dependables! !

!OpenVolunteerMarketplace methodsFor: 'changing' stamp: 'ThiagoFurtado 10/22/2012 14:53'!
addDependencyOn: dependable to: aTask

	aTask addDependencyOn: dependable .! !

!OpenVolunteerMarketplace methodsFor: 'initialize-release' stamp: 'pbm 10/20/2012 22:23'!
addFirstAdmin 
	database at: #volunteer add: (OVMPVolunteer new username: 'admin'; password: 'admin'; admin: true; name: 'Administrator').! !

!OpenVolunteerMarketplace methodsFor: 'adding' stamp: 'pbm 10/22/2012 15:51'!
addRequiredSkill: skill for: aTask coordinatedBy: aCoordinator 
	|tmpTask|
	tmpTask := self task: (aTask name).
	( tmpTask isNil ) 
		ifTrue: [
			self error: 'There is no task named ', aTask name.] 
		ifFalse: [
			self checkPermission: aTask with: aCoordinator.
			tmpTask addSkill: skill.
			] ! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/21/2012 20:21'!
applicationsBy: aVolunteer
	^database at: #application filter: #volunteer by: aVolunteer! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/21/2012 20:04'!
applicationsFor: aTask
	^database at: #application filter: #task by: aTask! !

!OpenVolunteerMarketplace methodsFor: 'changing' stamp: 'pbm 10/21/2012 19:39'!
candidate: aVolunteer toTask: aTask
	database at: #application add: (OVMPApplication to: aTask by: aVolunteer ).! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/21/2012 20:06'!
candidatesFor: aTask
	^((database at: #application filter: #task by: aTask) select: [:item | item status == #pending]) collect: [:item | item volunteer ]  ! !

!OpenVolunteerMarketplace methodsFor: 'changing' stamp: 'pbm 10/22/2012 15:52'!
changeStateOf: aTask toState: aSymbol by: aCoordinator
	|message|
	self checkPermission: aTask with: aCoordinator.
	message := Message new setSelector: aSymbol arguments: #(); yourself.
	message sendTo: aTask.
		! !

!OpenVolunteerMarketplace methodsFor: 'testing' stamp: 'pbm 10/22/2012 15:43'!
checkPermission: anObject with: aVolunteer
	((anObject coordinators includes: aVolunteer) or: aVolunteer isAdmin) ifFalse: [self error: 'permission denied']  ! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 01:51'!
cleanTasksOf: aProject
	(database collection: #task) select: [ :item | (aProject tasks includes: item) ifTrue: [database at: #task remove: item] ]! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/21/2012 14:07'!
database
	^ database.! !

!OpenVolunteerMarketplace methodsFor: 'testing' stamp: 'pbm 10/20/2012 22:15'!
existsUserIdentifiedBy: anUsername
	 (database at: #volunteer filter: #username by: anUsername ) isEmpty 
		ifTrue: [ ^ false. ] ifFalse: [^ true].! !

!OpenVolunteerMarketplace methodsFor: 'testing' stamp: 'pbm 10/20/2012 22:30'!
hasProject: aName
	^(database at: #project filter: #name by: aName) isEmpty not.! !

!OpenVolunteerMarketplace methodsFor: 'testing' stamp: 'pbm 10/20/2012 22:46'!
hasVolunteer: aUsername
	^(database at: #volunteer filter: #username by: aUsername) isEmpty not! !

!OpenVolunteerMarketplace methodsFor: 'initialize-release' stamp: 'ThiagoFurtado 10/22/2012 15:37'!
initialize 
	database := Database new.
	database create: #volunteer.
	database create: #project.
	database create: #task.
	database create: #skill.
	database create: #invitation.
	database create: #application.
	self addFirstAdmin.! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:15'!
invitationsTo: aVolunteer
	^database at: #invitation filter: #invitee by: aVolunteer ! !

!OpenVolunteerMarketplace methodsFor: 'creating' stamp: 'pbm 10/22/2012 15:52'!
invite: aVolunteer to: aTask by: aCoordinator
	self checkPermission: aTask with: aCoordinator.
	database at: #invitation add: (OVMPInvitation to: aVolunteer  for: aTask  by: aCoordinator).
! !

!OpenVolunteerMarketplace methodsFor: 'deprecated' stamp: 'ThiagoFurtado 10/21/2012 01:22'!
newProject: aName coordinatedBy: aCoordinator
	|project|
	(self hasProject: aName)  
		ifFalse: [
			project := OVMPProject forName: aName.
			project creator: aCoordinator.
			project addCoordinator: aCoordinator.  
			database at: #project add: project ]
		ifTrue:   [
			self error: 'project alreary exists'].
	^project ! !

!OpenVolunteerMarketplace methodsFor: 'creating' stamp: 'pbm 10/22/2012 13:24'!
newProjectName: name creator: creator url: url description: descr
	|project|
	(self hasProject: name)  
		ifFalse: [
			project := OVMPProject new name: name; creator: creator; addCoordinator: creator; url: url; description: descr yourself.
			database at: #project add: project ]
		ifTrue:   [
			self error: 'project alreary exists: ', name].
	^project ! !

!OpenVolunteerMarketplace methodsFor: 'creating' stamp: 'pbm 10/20/2012 22:16'!
newSkill: aSkillName withPermissionsOf: aVolunteer
	"Only admin can pass this message"
	|skill|
	(aVolunteer isAdmin ) ifTrue: [
		
		((self skill: aSkillName) isNil )
		ifFalse: [
				self error: 'skill name aready exists.'] 
		ifTrue: [
				skill := OVMPSkill forName: aSkillName.
				database at: #skill add: skill].
		^skill   
		
		] ifFalse: [self error: 'You have no permission to modify skills'] .
	! !

!OpenVolunteerMarketplace methodsFor: 'deprecated' stamp: 'pbm 10/22/2012 15:53'!
newTask: aName For: aProject coordinatedBy: aCoordinator 
	|task|	
	self checkPermission: aProject with: aCoordinator.
	task := (OVMPTask named: aName forProject: aProject startingAt: (Date today) ).
	database at: #task add: task.  
	^task.! !

!OpenVolunteerMarketplace methodsFor: 'creating' stamp: 'pbm 10/22/2012 15:54'!
newTaskName: name skills: skills project: aProject startDate: startDate estimatedDuration: duration estimatedConclusionDate: date withPermissionOf: aCoordinator 
	|task|	
	self checkPermission: aProject  with: aCoordinator.
	task := OVMPTask named: name forProject: aProject startingAt: startDate; addSkills: skills; estimatedDuration: duration; estimatedConclusionDate: date; yourself.   
	database at: #task add: task. 
	^task.
		! !

!OpenVolunteerMarketplace methodsFor: 'creating' stamp: 'pbm 10/22/2012 15:55'!
newTaskName: name skills: skills project: aProject startDate: startDate estimatedDuration: duration withPermissionOf: aCoordinator 
	|task|	
	self checkPermission: aProject with: aCoordinator.
	task := (OVMPTask named: name forProject: aProject startingAt: startDate) addSkills: skills; estimatedDuration: duration; yourself.   
	database at: #task add: task.  
	^task.
		! !

!OpenVolunteerMarketplace methodsFor: 'deprecated' stamp: 'pbm 10/20/2012 22:48'!
newVolunteer: aUsername
	|volunteer|
	 (self hasVolunteer: aUsername )  
		ifTrue: [
				self error:'username aready in use:', aUsername ] 
		ifFalse: [
				volunteer := OVMPVolunteer forUsername: aUsername.
				database at: #volunteer add: volunteer].
	^volunteer    ! !

!OpenVolunteerMarketplace methodsFor: 'creating' stamp: 'ThiagoFurtado 10/22/2012 13:36'!
newVolunteerUsername: username password: password email: email name: name birthday: birthday skills: skills isAdmin: bool
	|volunteer|
	 (self hasVolunteer: username )  
		ifTrue: [	self error:'username aready in use: ', username.
				^ nil] 
		ifFalse: [
				volunteer := OVMPVolunteer new username: username; password: password; email: email; email: email; name: name; birthday: birthday; addSkills: skills; admin: bool; yourself.
				database at: #volunteer add: volunteer].
	^volunteer    ! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:16'!
project: aName
	|project|
	project := (database at: #project filter: #name by: aName).
	
	( project isEmpty ) 
		ifTrue: [ ^nil ]
		ifFalse: [ project select: [:item | (item name = aName) ifTrue: [ ^ item ] ] ] 
		! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/22/2012 15:04'!
reject: anInvitation by: anInvitee
	(anInvitation invitee = anInvitee) ifFalse: [self error:'operation not allowed'].
	anInvitation reject.
	! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 20:00'!
reject: anInvitation reason: aComment by: anInvitee
	(anInvitation invitee = anInvitation) ifFalse: [self error:'operation not allowed'].
	anInvitation reject: aComment.
	! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/22/2012 15:55'!
rejectApplication: anApplication reason: aComment withPermissionsOf: aCoordinator
	self checkPermission: anApplication task with: aCoordinator.
	anApplication reject: aComment.
	! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/22/2012 15:56'!
rejectApplication: anApplication withPermissionsOf: aCoordinator
	self checkPermission: anApplication task with: aCoordinator.
	anApplication reject.
	! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/22/2012 17:08'!
removeDependenciesOf: task
	|tasks each |
	tasks := task project tasks.
	tasks do: [:item | 
		( (item dependsOn) includes: task) 
			ifTrue: [item dependsOn remove: task ] ] .! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'pbm 10/20/2012 22:17'!
removeInvitationsFor: aTask
		database at: #invitation removeAll: (database at: #invitation filter: #task by: aTask)
! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 01:51'!
removeProject: aProject withPermissionsOf: aVolunteer
	( aVolunteer isAdmin )
		ifTrue: [	self cleanTasksOf: aProject.
			"(database collection: #task) select: [:item | (aProject tasks includes: item) ifTrue: [database at: #task remove: item] ]".
				database at: #project remove: aProject]
		ifFalse: [
			"Any creator can remove a project."
			( aProject creator = aVolunteer )
				ifTrue: [
					database at: #project remove: aProject ].
			]! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 01:27'!
removeProjectNamed: aName withPermissionsOf: aVolunteer
	self removeProject: (self project: aName) withPermissionsOf: aVolunteer .! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'pbm 10/20/2012 22:17'!
removeSkill: aName withPermissionsOf: aVolunteer
	(aVolunteer isAdmin ) ifTrue: [database at: #skill remove: aName. ] ifFalse: ['You have no permission to modify skills'].! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/20/2012 21:00'!
removeSkillNamed: aName withPermissionsOf: aVolunteer
	self removeSkill: (self skill: aName) withPermissionOf: aVolunteer.! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'pbm 10/22/2012 15:57'!
removeTask: aTask from: aProject withPermissionsOf: aCoordinator
	self checkPermission: aTask with: aCoordinator.
	aProject tasks remove: aTask.
	self removeTasks: ( (Set new) add: aTask; yourself) 
	! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/22/2012 15:25'!
removeTasks: aCollection
	aCollection do:[:task | self removeInvitationsFor: task].
	aCollection do:[:task | self removeDependenciesOf: task].
		database at: #task removeAll: aCollection.! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'pbm 10/22/2012 16:00'!
removeVolunteer: aVolunteer from: aTask withPermissionsOf: aCoordinator
	 "Removes a volunteer from a task, but not from application"
	(aVolunteer = aCoordinator) ifFalse: [self checkPermission: aTask with: aCoordinator.].  
	aTask volunteers remove: aVolunteer
		! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'pbm 10/20/2012 22:17'!
removeVolunteer: aVolunteer withPermissionsOf: me
	"only me or admin can remove from application"
	(aVolunteer = me) 
		ifTrue: [
			database at: #volunteer remove: aVolunteer. ]
		ifFalse: [
			(me isAdmin) 
				ifTrue: [ 
					database at: #volunteer remove: aVolunteer. ] 
				ifFalse: 
					[ self error: 'You can only remove yourself' ] ]! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 12:44'!
removeVolunteerNamed: aName from: aTask withPermissionsOf: aCoordinator
	self removeVolunteer: (self volunteer: aName) from: aTask withPermissionsOf: aCoordinator . ! !

!OpenVolunteerMarketplace methodsFor: 'removing' stamp: 'ThiagoFurtado 10/21/2012 12:37'!
removeVolunteerNamed: aName withPermissionsOf: aVolunteer
	"only me or admin can remove from application"
	self removeVolunteer: (self volunteer: aName) withPermissionsOf: aVolunteer.! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:18'!
skill: aName
	|skill|
	skill := (database at: #skill filter: #name by: aName).
	( skill isEmpty ) 
		ifTrue: [ ^nil ]
		ifFalse: [ skill select: [:item | (item name = aName) ifTrue: [ ^ item ] ] ] 
		! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 23:47'!
suggestedTasks: aVolunteer
	"suggests the open tasks that require only known skills"
	^(database at: #task filter: #isOpen by: true) select: [:task| (task checkUncoveredSkills: aVolunteer skills) isEmpty ]! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:18'!
suggestedVolunteers: aTask
	"suggests the volunteers who have all the required skills for aTask"
	^(database collection: #volunteer)  select: [:volunteer | (aTask checkUncoveredSkills: volunteer skills) isEmpty ].! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:18'!
task: aName
	|task|
	task := (database at: #task filter: #name by: aName).
	( task isEmpty ) 
		ifTrue: [ ^nil ]
		ifFalse: [ task select: [:item | (item name = aName) ifTrue: [ ^ item ] ] ] 
		! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/21/2012 14:05'!
tasks
	^ self database collection: #task.! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'ThiagoFurtado 10/21/2012 16:18'!
tasksOf: aProject
	^ aProject tasks.! !

!OpenVolunteerMarketplace methodsFor: 'accessing' stamp: 'pbm 10/20/2012 22:18'!
volunteer: anUsername
	|users|
	users := (database at: #volunteer filter: #username by: anUsername).
	
	( users isEmpty ) ifTrue: [ ^ nil ]
		ifFalse: [ users select: [:item | (item username = anUsername) ifTrue: [ ^ item ] ]  ]
		
	
	! !
OVMPView initialize!
